@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.24 2001/07/13 10:06:26 bazsi Exp $ */
+ * $Id: io.c,v 1.25 2001/08/26 21:28:18 bazsi Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -444,7 +444,8 @@ static void read_callback(struct nonblocking_fd *fd)
        (name fd_write)
        (super abstract_write)
        (vars
-         (fd simple int)))
+         (fd simple int)
+         (fsync simple int)))
 */
 
 static int do_write(struct abstract_write *w, UINT32 length, UINT8 *data)
@@ -473,6 +474,8 @@ static int do_write(struct abstract_write *w, UINT32 length, UINT8 *data)
 			       errno, strerror(errno));
 		}
 	}
+	if (self->fsync)
+		fsync(self->fd);
 	return res;
 }
 
@@ -482,7 +485,7 @@ static void write_callback(struct nonblocking_fd *fd)
 	int res;
 
 	struct fd_write w =
-	{ { STACK_HEADER, do_write }, fd->fd };
+	{ { STACK_HEADER, do_write }, fd->fd, self->fsync };
 
 	assert(self->buffer);
 
@@ -562,6 +565,9 @@ static void prepare(struct nonblocking_fd *fd)
 {
 	CAST(io_fd, self, fd);
 
+	if (self->buffer && fd->to_be_closed == 1) 
+		BUF_CLOSE(self->buffer);
+		
 	if (fd->super.alive && self->buffer)
 		fd->want_write = BUF_PREPARE(self->buffer);
 	else
@@ -569,7 +575,6 @@ static void prepare(struct nonblocking_fd *fd)
 	/* attempt to flush data 3 times, before fd is really closed */
 	if (fd->to_be_closed) {
 		if (fd->want_write) {
-			if (self->buffer && fd->to_be_closed == 1) BUF_CLOSE(self->buffer);
 			if (fd->to_be_closed > 3)
 				kill_fd(fd);
 			fd->to_be_closed++;
@@ -619,7 +624,7 @@ static void do_kill_fd(struct resource *r)
 	 * killed files. For now, using the zero reason will supress calling
 	 * of any close callbacks. */
 	if (r->alive)
-		closekill_fd(fd, 0);
+		close_fd(fd, 0);
 }
 
 /* Initializes a file structure, and adds it to the backend's list. */
